import mapeathor

from flask import Flask, send_file, jsonify
from flask_swagger import swagger
from flask_swagger_ui import get_swaggerui_blueprint
from flask_restful import Resource, Api, reqparse
from werkzeug import datastructures
import tempfile

parser = reqparse.RequestParser()
parser.add_argument('file',type=datastructures.FileStorage, location='files')
parser.add_argument('url')
parser.add_argument('format')

app = Flask(__name__)
api = Api(app)

# Input:
## file: Excel file
### [or]
## url: Google Drive

## format: output mapping format

# Output:
## file: output mapping

SWAGGER_URL = '/api/docs'  # URL for exposing Swagger UI (without trailing '/')
API_URL = '/api/'  # Our API url (can of course be a local resource)

# Call factory function to create our blueprint
swaggerui_blueprint = get_swaggerui_blueprint(
    SWAGGER_URL,  
    API_URL,
    config={ 
        'app_name': "Mapeathor"
    },
)

app.register_blueprint(swaggerui_blueprint, url_prefix=SWAGGER_URL)


class Mapeathor(Resource):
    def get(self):
        swag = swagger(app)
        swag['info']['version'] = "1.0.0"
        swag['info']['title'] = "Mapeathor"
        return jsonify(swag)
        
    def post(self):
        
        """
        Translate mapping document
        ---
        
        components:
          schemas:
            Excel:
              type: object
              properties:
                format:
                  type: string
                  enum: [ "rml", "r2rml", "yarrrml"]
                file:
                  type: file
                  description: XLS[X] file to convert
            GSpreadSheet:
              type: object
              properties:
                format:
                  type: string
                  enum: [ "rml", "r2rml", "yarrrml"]
                url:
                  type: string
                  description: Google Spreadsheet url to convert

        summary: Uploads a file.
        consumes:
            - multipart/form-data
        produces:
            - binary/octet-stream
        parameters:
            - in: formData
              name: format
              type: text
              enum: [ "rml", "r2rml", "yarrrml"]
              description: Mapping output format
            - in: formData
              name: url
              type: string
              description: Google Spreadsheet url to convert
            - in: formData
              name: file
              type: file
              description: XLS[X] file to convert
        responses:
            '200':
                description: 'Mapping file in the specified format'
            '401':
                description: 'Bad request'
            '500':
                description: 'Internal tool error'
        """
        
        data = parser.parse_args()
        
        print(data)
        
        if data['file'] is None and data['url'] is None:
            return {'message':'File or URL not found'}
        if data['format'] is None:
            return {'message':'No output mapping format specified'}
        else:
            
            temp_out = tempfile.NamedTemporaryFile(prefix="mapeathor-api").name
            
            if data['file'] is None or data['file'].filename == "":
                
                temp_in = mapeathor.gdriveToXMLX(data['url'])
            
            else:
                
                temp_in = tempfile.NamedTemporaryFile(prefix="mapeathor-api").name

                data['file'].save(temp_in)
                
                print(len(temp_in))
            
            mapeathor.setMappingLanguage(data["format"])
            
            outputFile = mapeathor.generateMapping(temp_in, temp_out)
            
            return send_file(outputFile, as_attachment=True)
            

api.add_resource(Mapeathor, API_URL)


def run(host="0.0.0.0", port=5000):
    
    print("API URL: "+API_URL)
    print("SWAGGER URL: "+SWAGGER_URL)
    
    app.run(debug=False, host=host, port=port)

if __name__ == '__main__':
    run()




import tempfile
import pkgutil
import os


tmpDir = tempfile.TemporaryDirectory(prefix="mapeathor").name+"/"
baseTemplatesDir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates') + '/'
dataTypesFile = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'dataTypes.json')
resultDir = tempfile.TemporaryDirectory(prefix="mapeathor").name+"/"
supportedLanguages = {'rml', 'r2rml', 'yarrrml'}


def setMappingLanguage(language):
    global templatesDir
    templatesDir = baseTemplatesDir + language.lower() + "/"


defaultDataTypes = {

   "string":[
      "string",
      "nan",
      " ",
      ""
   ],
   "decimal":[
      "decimal"
   ],
   "float":[
      "float"
   ],
   "double":[
      "double"
   ],
   "integer":[
      "integer",
      "int",
      "number"
   ],
   "boolean":[
      "boolean",
      "bool"
   ],
   "date":[
      "date",
   ],
   "time":[
      "time"
   ],
   "dateTime":[
      "datetime"
   ],
   "anyURI":[
      "anyuri",
      "iri",
      "uri",
      "url"
   ]
}



import os
import json
from jinja2 import Environment, FileSystemLoader

from . import global_config
from . import utils


def writeValues(data, path):
    """
    Redirects the data to the specific functions to write the 'data' into the file 'path'
    """
    if not os.path.isdir(global_config.tmpDir):
        os.mkdir(global_config.tmpDir)

    writePrefix(data,path)
    for triplesmap in data['TriplesMap']:
        writeTriplesMap(triplesmap, path)
        writeSubject(data['TriplesMap'][triplesmap]['Subject'], path)
        writeSource(data['TriplesMap'][triplesmap]['Source'], path)
        writePredicateObjects(data['TriplesMap'][triplesmap]['Predicate_Object'], path)

    # Functions implemented only in RML
    if global_config.templatesDir[-5:-1] == '/rml':
        for function in data['Function']:
            writeFunctionMap(function, path)
            writeFunctionPOM(data['Function'][function]['Predicate_Object'], path)
            writeFunctionSource(data['Function'][function]['Source'], path)


def writePrefix(data, path):
    """
    Writes the prefixes temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)
    base_template = env.get_template('Base.tmpl')
    prefix_template = env.get_template("Prefix.tmpl")

    for prefix in data['Prefix']:

        if ':' in str(prefix['Prefix']):
            re.sub(':', '', str(prefix['Prefix']))
        elif prefix['Prefix'] == 'nan' or prefix['URI'] == 'nan':
            continue

        if prefix['Prefix'] == '@base':
            base_output = base_template.render(prefixes=prefix)
            f = open(global_config.tmpDir + 'Base.txt', 'w')
            f.write(base_output + "\n")
            f.close()
            writeResult('', 'Base')

        else:
            prefix_output = prefix_template.render(prefixes=prefix)
            f = open(global_config.tmpDir + 'Prefix.txt', 'w')
            f.write(prefix_output + "\n")
            f.close()
            writeResult('', 'Prefix')


def writeTriplesMap(data, path):
    """
    Writes the triples map temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)
    template = env.get_template('TriplesMap.tmpl')

    triples_maps = {'ID' : str(data)}
    output = template.render(tm=triples_maps)
    f = open(global_config.tmpDir + 'TriplesMap.txt', 'w')
    f.write(output + "\n")
    f.close()
    writeResult(str(data), 'TriplesMap')


def writePredicateObjects(data, path):
    """
    Writes the Predicate_Object temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)

    for key in data:
        if(len(data[key]) > 0):
            for predicateObjects in data[key]:
                template = env.get_template(key + '.tmpl')

                predicateObjects['Object'] = utils.replaceVars(str(predicateObjects['Object']), str(predicateObjects['ObjectType']), str(predicateObjects['TermType']))
                predicateObjects['Predicate'] = utils.replaceVars(str(predicateObjects['Predicate']), str(predicateObjects['PredicateType']), 'nan')
                if( 'InnerRef' in predicateObjects.keys() and 'OuterRef' in predicateObjects.keys()):
                    predicateObjects['InnerRef'] = utils.replaceVars(str(predicateObjects['InnerRef']), 'join_condition', 'nan')
                    predicateObjects['OuterRef'] = utils.replaceVars(str(predicateObjects['OuterRef']), 'join_condition', 'nan')

                output = template.render(pom=predicateObjects)
                f = open(global_config.tmpDir + key + '.txt', 'w')
                f.write(output + "\n")
                f.close()
                writeResult(data[key][0]['ID'], key)


def writeSource(data, path):
    """
    Writes the source temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)

    config  = json.loads(open(global_config.templatesDir + 'config.json').read())
    if(data['Iterator'] != ''):
        data['Iterator'] = str(config['iterator']['before']) + str(data['Iterator']) + str(config['iterator']['after'])

    if('Query' in data.keys()):
        template = env.get_template('SourceQuery.tmpl')
    else:
        template = env.get_template('Source.tmpl')

    output = template.render(source=data)
    f = open(global_config.tmpDir + 'Source.txt', 'w')
    f.write(output + "\n")
    f.close()
    writeResult(data['ID'], 'Source')


def writeSubjectTemp(data, path):
    """
    NOT IN USE
    USES GO_TEMPLATE -- Now jinja is used
    Writes the subject temporal file from the template with the information in 'data' into the path 'path'
    """
    f = open(path + 'Subject.yml', 'a+')
    data['URI'] = utils.replaceVars(data['URI'], data['SubjectType'], 'nan')
    for element in data:
        if element != 'Class':
            f.write(element + ': ' + data[element] + '\n')
        else:
            for i in range(0, len(data[element])):
                f.write(element + str(i) + ': ' + data[element][i] + '\n')
    f.close()
    go_template.render_template(global_config.templatesDir + 'Subject.tmpl',global_config.tmpDir + 'Subject.yml', global_config.tmpDir + 'Subject.txt')
    writeResult(data['ID'], 'Subject')


def writeSubject(data, path):
    """
    Writes the subject temporal file from the template with the information in 'data' into the path 'path'
    """

    f = open(global_config.tmpDir + 'Subject.txt', 'a+')

    # Evaluate if URI is nan, because after it doesn't work calling utils.replaceVars
    if data['URI'] == 'nan':
        isnan = True
    else:
        isnan = False

    data['URI'] = utils.replaceVars(data['URI'], data['SubjectType'], 'nan')
    data['SubTermMap'] = utils.replaceTermMap(data['SubjectType'])

    if global_config.templatesDir[-8:-1] != 'yarrrml':
        f.write('rr:subjectMap [\n\ta rr:Subject;\n\trr:termType rr:' + data['SubjectTermType'] + ';\n')
        if not isnan:
            f.write('\t' + data['SubTermMap'] + ' ' + data['URI'] + ';\n')

        for class_s in data['Class']:
            f.write('\trr:class ' + class_s + ';\n')
        f.write('];\n')
    else:
        if not isnan:
            f.write('s: ' + data['URI'] + '\n')
        f.write('po:\n')
        for class_s in data['Class']:
            f.write('  - [a, ' + class_s + ']\n')

    f.close()
    writeResult(data['ID'], 'Subject')


def writeFunctionMap(data, path):
    """
    Writes the function map temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)
    template = env.get_template('FunctionMap.tmpl')

    fun_tm = {'FunctionID' : data}
    output = template.render(fun_tm=fun_tm)
    f = open(global_config.tmpDir + 'FunctionMap.txt', 'w')
    f.write(output + "\n")
    f.close()
    writeResult(str(data), 'FunctionMap')


def writeFunctionSource(data, path):
    """
    Writes the source of function temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)
    template = env.get_template('FunctionSource.tmpl')

    config  = json.loads(open(global_config.templatesDir + 'config.json').read())
    if(data['Iterator'] != ''):
        data['Iterator'] = str(config['iterator']['before']) + str(data['Iterator']) + str(config['iterator']['after'])

    output = template.render(source=data)
    f = open(global_config.tmpDir + 'FunctionSource.txt', 'w')
    f.write(output + "\n")
    f.close()
    writeResult(data['FunctionID'], 'FunctionSource')


def writeFunctionPOM(data, path):
    """
    Writes the Predicate_Object of function temporal file from the template with the information in 'data' into the path 'path'
    """
    file_loader = FileSystemLoader(global_config.templatesDir)
    env = Environment(loader=file_loader)
    template = env.get_template('FunctionPOM.tmpl')

    for pom in data:
        if pom['Feature'] != 'fno:executes' and str(pom['Value'])[0] != '<':
            pom['Value'] = '\"' + pom['Value'] + '\"'

        output = template.render(pom=pom)
        f = open(global_config.tmpDir + 'FunctionPOM.txt', 'w')
        f.write(output + "\n")
        f.close()
        writeResult(pom['FunctionID'], 'FunctionPOM')


def writeResult(ID, name):
    """
    Writes the temporal files assigning ID and names for the different parts of the mapping
    """
    delete = open(global_config.tmpDir + name + '.txt', 'r')
    final = open(global_config.resultDir + ID + '.' + name + '.' + 'result.txt', 'a+')
    final.writelines(delete.readlines())
    delete.close()
    final.close()

    try:
        os.remove(global_config.tmpDir + name + '.txt')
        os.remove(global_config.tmpDir + name + '.yml')
    except:
        pass


def writeFinalFile(path_, idTMList, idFList):
    """
    Gathers all the temporal files identified with 'idTMList' and 'idFList' and writes
    it in the final mapping file in 'path_'
    """
    data = json.loads(open(global_config.templatesDir  + 'structure.json').read())
    config = json.loads(open(global_config.templatesDir + 'config.json').read())

    if not path_.endswith(str(config['extension'])):
        path = path_ + '.' +  str(config['extension'])
    else:
        path = path_

    if os.path.isfile(path):
        os.remove(path)

    recursiveWrite(0,data['unique'], path, '')
    for id_ in idTMList:
        recursiveWrite(0, data['variable'], path, id_)
    for id_ in idFList:
        recursiveWrite(0, data['variable'], path, id_)
    return path


def recursiveWrite(tabs, parent, finalFile, id_):
    """
    Writes the temporal files in one recursively
    """
    for data in range(0, len(parent)):
        file_ = global_config.resultDir + id_ + '.' + parent[data]['file'] + '.result.txt'
        config = json.loads(open(global_config.templatesDir + 'config.json').read())
        exists = os.path.isfile(file_)
        if(exists):
            f = open(file_, 'r')
            final = open(finalFile, 'a+')
            if(str(parent[data]["before"]) != ""):
                final.write(parent[data]["before"] + '\n')
            for line in f.readlines():
                final.write(' ' * int(config['tab']['size']) * tabs + str(line))
            f.close()
            #os.remove(file_)
            final.close()
            if(len(parent[data]['childs']) > 0):
                recursiveWrite(parent[data]['tabs'], parent[data]['childs'], finalFile, id_)
            if(str(parent[data]["after"]) != ""):
                final = open(finalFile, 'a+')
                final.write(parent[data]["after"] + '\n')
                final.close()



import sys
import argparse

from . import global_config
from . import source
from . import mapping_generator
from ._version import __version__


def main():
    parser = argparse.ArgumentParser("mapeathor")
    parser.add_argument("-i", "--input_file", required=True, help="Input Excel file or Google SpreadSheet URL")
    parser.add_argument("-o", "--output_file", required=False, help="Name and path for output file", default="output")
    parser.add_argument("-l", "--language", required=True, help=("Supported Languages: " + str(global_config.supportedLanguages)))
    parser.add_argument("-v", "--version", help=("Version"), action="version", version="Mapeathor " + __version__)
    args = parser.parse_args()
    inputFile = ''

    # Local file
    if(source.checkFile(args.input_file)):
        inputFile = str(args.input_file)
    elif (args.input_file[0:8] != 'https://'):
        print('ERROR: File not found')
        sys.exit()

    # Google Spreadsheet file
    else:
        temp = source.gdriveToXMLX(args.input_file)
        if source.checkFile(temp):
            inputFile = temp
        else:
            print("ERROR: The downloaded document is not a spreadsheet")
            sys.exit()

    # Wrong language or not supported
    if(args.language.lower() not in global_config.supportedLanguages):
        print("ERROR: The selected Language is not supported by the moment.")
        print("Supporteds Languages: " + str(global_config.supportedLanguages))
        sys.exit()
    else: # Create mapping
        global_config.setMappingLanguage(args.language)
        print('Generating mapping file')
        outputFile = mapping_generator.generateMapping(inputFile, args.output_file)
        print("Your mapping file is in "+outputFile)

if __name__ == '__main__':
    main()



import json
import re

from . import global_config


def replaceVars(element, type_, termtype_):
    """
    Writes the objects 'element' correctly according to their type 'type_' and termtype 'termtype_', returns the
    corrected object
    """
    config = json.loads(open(global_config.templatesDir + 'config.json').read())
    # Add "" to the constant literal objects in YARRRML
    if(global_config.templatesDir[-8:-1] == 'yarrrml' and type_ == 'constant' and termtype_ == 'Literal'):
        result = str(config['variable'][type_]['before']) + element + str(config['variable'][type_]['after'])

    # Replace '{' and '}' in all but constant objects according to the config.json file of each language
    elif(type_ != 'constant' and str(config['variable'][type_]['before']) != '{' and str(config['variable'][type_]['after']) != '}'):
        result = element.replace("{", str(config['variable'][type_]['before'])).replace("}", config['variable'][type_]['after'])

    elif(((termtype_ != 'IRI' and type_ == 'constant') or type_ == 'template') and (global_config.templatesDir[-4:-1] == 'rml' or global_config.templatesDir[-6:-1] == 'r2rml')):
        result = "\"" + element + "\""

    else:
        result = element
    return result


def replaceTermMap(type_):
    config = json.loads(open(global_config.templatesDir + 'config.json').read())
    result = config['variable'][type_]['termMap']
    return result


def dataTypeIdentifier(element):
    """
    Identifies the datatype of the object 'element' and returns it
    """
    # Load the json with some xsd datatypes predefined

    try:
        data = open(global_config.dataTypesFile).read()
        dataTypes = json.loads()
    except:
        dataTypes = global_config.defaultDataTypes

    for key in dataTypes.keys():
        if element.lower().strip() in dataTypes[key]:
            return key

    print('WARNING: datatype not recognized (' + element + '), check XSD datatypes')
    return element


def termTypeIdentifier(element, dataType):
    """
    Identifies the termtype of the object 'element' based on itself and its datatype 'dataType' and returns it
    """
    if(len(str(element).split(":")) == 2 or "http" in str(element) or dataType == "anyURI"):
        return 'IRI', '~iri'
    else:
        return 'Literal', ''

def subjectTermTypeIdentifier(element):
    """
    Identifies the termtype of the subject 'element' based on if it is formatted as an IRI or not
    """
    if(len(str(element).split(":")) == 2 or "http" in str(element)):
        return 'IRI'
    elif element == 'nan':
        return 'BlankNode'
    else:
        return 'BlankNode'


def predicateTypeIdentifier(element):
    """
    Identifies the type of the predicate, distinguishing between constant, reference and template, and returns it
    """
    # For constant
    if(len(str(element).split(":")) == 2 and "{" not in str(element) and "}" not in str(element)):
        return 'constant'
    # For reference
    elif(str(element)[:1] == '{' and str(element)[-1:] == '}' and len(str(element).split(" "))  == 1):
        return 'reference'
    # For template
    elif(bool(re.search("{.+}.+", str(element))) or bool(re.search(".+{.+}", str(element)))):
        return 'template'
    # Constant when not recognized
    else:
        #print("WARNING: type not identified for predicate '" + element + "', 'constant' assigned")
        return 'constant'


def cleanDir(path):
    """
    Cleans the results directory
    """
    dir = os.listdir(path)
    for f in dir:
        os.remove(path + f)



import pandas
import sys
import os
import json
import re
import copy

from . import global_config
from . import mapping_writer
from . import utils

def generateJson(path):
    """
    The input excel file 'path' is translated into JSON format, returns the json
    """
    data = pandas.ExcelFile(path, engine='openpyxl')
    json = {}
    for sheet_ in data.sheet_names:
        sheet = str(sheet_)
        json[sheet] = {}
        json[sheet] = generateJsonCols(data.parse(sheet))
    return json


def generateJsonCols(data):
    """
    Arranges correctly the input pandas dataframe into a json, which is returned as output
    """
    rng = len(data[data.columns[0]])
    result = []
    for row in range(0, rng):
        element = {}
        for col_ in data.columns:
            col = str(col_)
            element[col] = str(data[col][row])
        result.append(element)
    return result


def organizeJson(data):
    """
    Rearranges the data in JSON format into the desired structure, returns the resulting json
    """
    json = {}
    json['Prefix'] = data['Prefix']
    json['TriplesMap'] = {}
    formated_subjects = reFormatSubject(data['Subject'])
    for subject in formated_subjects:
        json['TriplesMap'][subject['ID']] = findChilds(data, subject['ID'])
        json['TriplesMap'][subject['ID']]['Subject'] = subject
        json['TriplesMap'][subject['ID']]['Subject']['SubjectType'] = utils.predicateTypeIdentifier(subject['URI'])
        json['TriplesMap'][subject['ID']]['Subject']['SubjectTermType'] = utils.subjectTermTypeIdentifier(subject['URI'])
        json['TriplesMap'][subject['ID']]['Source'] = reFormatSource(json['TriplesMap'][subject['ID']]['Source'])
        json['TriplesMap'][subject['ID']]['Predicate_Object']  =  reFormatPredicateObject(json['TriplesMap'][subject['ID']]['Predicate_Object'])
    json['Function'] = reFormatFunction(data['Function'], json)
    return json


def reFormatSubject(data):
    """
    Reorganize subjects in case there is more than one class per ID
    """
    id_class = {}
    for element in data:
        if element['ID'] == 'nan':
            continue
        elif element['ID'] in id_class:
            id_class[element['ID']].append(element['Class'])
        else:
            id_class[element['ID']] = [element['Class']]

    subjects = []
    for element in data:
        if element['ID'] in id_class:
            element['Class'] = id_class[element['ID']]
            del id_class[element['ID']]
            subjects.append(element)

    return(subjects)


def findChilds(data, ID):
    """
    Finds the Predicate_Object and Source in 'data' of the given 'ID' and returns it
    """
    result = {}
    keys = sorted(data.keys())
    keys.remove('Subject')
    keys.remove('Prefix')
    keys.remove('Function')

    for key in keys:
        result[key] =  []
        for element in data[key]:
            if(element['ID'] == ID):
                result[key].append(element)
    return result


def reFormatFunction(data_function, data):
    """
    Rearranges the data of functions 'data_function' into the desired structure and adds the source of each function
    with 'data'. The rearranged json is returned.
    """
    # Empty sheet
    if data_function == []:
        return({})
    elif data_function[0]['Feature'] == 'nan' and data_function[0]['Value'] == 'nan':
        return({})

    result = {}
    for element in data_function:
        # Blank rows
        if element['FunctionID'] == 'nan':
            continue

        element['FunctionID'] = str(element['FunctionID'])[1:-1]
        if element['FunctionID'] not in result.keys():
            result[element['FunctionID']] = {'Predicate_Object':[], 'Source':[]}
            FID = element['FunctionID']

        if element['FunctionID'] == FID:
            if("{" not in str(element['Value']) and "}" not in str(element['Value']) and '<' != str(element['Value'])[0]):
                element['ValueType'] = 'rr:constant'
            elif(str(element['Value'])[:1] == '{' and str(element['Value'])[-1:] == '}'):
                element['Value'] = str(element['Value'])[1:-1]
                element['ValueType'] = 'rml:reference'
            elif(str(element['Value'])[:1] == '<' and str(element['Value'])[-1:] == '>'):
                element['Value'] = '<#' + str(element['Value'])[1:]
                element['ValueType'] = ''
            else:
                print('WARNING: Wrong element in Function', FID)
                element['ValueType'] = 'rr:constant'
            result[element['FunctionID']]['Predicate_Object'].append(element)

    for fun in result:
        result[fun]['Source'] = find_source(fun, data, result)
        result[fun]['Source']['FunctionID'] = fun
        #print(result['Fun1']['Source'])
    return(result)


def find_source(function_key, data, functions):
    """
    Finds the source of the 'function_key' in 'data' or in 'functions', and returns it
    """
    for tm in data['TriplesMap']:
        if len(data['TriplesMap'][tm]['Predicate_Object']['Function']) != 0:
            for fun in data['TriplesMap'][tm]['Predicate_Object']['Function']:
                if fun['Object'] == function_key:
                    #data['TriplesMap'][tm]['Source']['FunctionID'] = function_key
                    #print(data['TriplesMap'][tm]['Source'])
                    return(data['TriplesMap'][tm]['Source'])
    for fun in functions:
        if len(functions[fun]['Source']) != 0:
            for element in functions[fun]['Predicate_Object']:
                if element['Value'][2:-1] == function_key:
                    functions[function_key]['Source'] = functions[fun]['Source'].copy()
                    #functions[function_key]['Source']['FunctionID'] = function_key
                    return(functions[function_key]['Source'])


def reFormatPredicateObject(data):
    """
    Rearranges the json contained in 'data' for the Triple Object Maps, and returns it
    """
    result = {'Join':[], 'Function':[], 'POM':[]}
    nullValues =  {'', 'NaN', ' ', 'nan', 'NAN'}
    for element in data:
        element['PredicateType'] = utils.predicateTypeIdentifier(element['Predicate'])
        element['PredTermMap'] = utils.replaceTermMap(element['PredicateType'])
        element['DataType'] = utils.dataTypeIdentifier(element['DataType'])
        element['TermType'], element['isIRI'] = utils.termTypeIdentifier(element['Object'], element['DataType'])

        # Populate Join key
        if(str(element['Object'])in nullValues and str(element['InnerRef']) not in nullValues and str(element['OuterRef']) not in nullValues):
            element['ObjectType'] = 'reference'
            result['Join'].append(element)
        # Populate Function key
        elif(str(element['Object'])[:1] == '<' and str(element['Object'])[-1:] == '>'):
            element['ObjectType'] = 'reference'
            element['Object'] = str(element['Object'])[1:-1]
            result['Function'].append(element)
        # Populate Constant key
        elif("{" not in str(element['Object']) and "}" not in str(element['Object'])):
            element['ObjectType'] = 'constant'
            element['ObjTermMap'] = utils.replaceTermMap(element['ObjectType'])
            result['POM'].append(element)
        # Populate Template key
        elif(bool(re.search("{.+}.+", str(element['Object']))) or bool(re.search(".+{.+}", str(element['Object'])))):
            element['ObjectType'] = 'template'
            element['ObjTermMap'] = utils.replaceTermMap(element['ObjectType'])
            result['POM'].append(element)
        # Populate Reference key when none of the conditions above are fulfilled
        else:
            element['ObjectType'] = 'reference'
            element['ObjTermMap'] = utils.replaceTermMap(element['ObjectType'])
            result['POM'].append(element)
    return result


def reFormatSource(data):
    """
    Rearranges the format of the 'Source' sheet in 'data' and returns it
    """
    result = {}
    for element in data:
        if(element['Feature'].lower() == 'source'):
            result['Source'] = str(element['Value'])
        elif(element['Feature'].lower() == 'format'):
            result['Format'] = str(element['Value'])
            if result['Format'].lower() == 'json':
                result['Format'] = 'JSONPath'
            elif result['Format'].lower() == 'xml':
                result['Format'] = 'XPath'
            elif result['Format'].lower() == 'xquery':
                result['Format'] = 'XQuery'
            elif result['Format'].lower() == 'csv':
                result['Format'] = 'CSV'

            if global_config.templatesDir[-8:-1] == 'yarrrml':
                result['Format'] = str(result['Format']).lower()

        elif(element['Feature'].lower() == 'iterator'):
            result['Iterator'] = str(element['Value'])
        elif(element['Feature'].lower() == 'table'):
            result['Table'] = str(element['Value'])
        elif(element['Feature'].lower() == 'query'):
            result['Query'] = str(element['Value'])
        elif(element['Feature'].lower() == 'sqlversion'):
            result['SQLVersion'] = str(element['Value'])
        else:
            print("ERROR: " + element + " feature not recognized. The recognized values for the \
                column 'Feature' in the sheet 'Source' are 'source', 'format', 'iterator', \
                'table', 'query' and 'SQLVersion")
            sys.exit()

    if('Iterator' not in result.keys()):
        result['Iterator'] = ''
    if('SQLVersion' not in result.keys()):
        result['SQLVersion'] = 'SQL2008'
    if('Source' not in result.keys() and 'Table' in result.keys()):
        result['Source'] = result['Table']
        if ('Format' not in result.keys()):
            result['Format'] = result['SQLVersion']


    result['ID'] = data[0]['ID']
    return result


def generateMapping(inputFile, outputFile=None):
    """
    General function with the flow of the script to generate and organize the JSON file with the data from
    'inputFile', and write it in the mapping file
    """
    if os.path.isdir(global_config.resultDir):
        utils.cleanDir(global_config.resultDir)
    else:
        os.mkdir(global_config.resultDir)

    if outputFile is None:
        outputFile = global_config.resultDir + re.findall(r'\/?([\w\-\_\[\]\(\)]+)\.',inputFile)[0]  ## wider option \/?([^\.\/]+)\.

    ## Without try/except
    #json = generateJson(inputFile)
    #json = organizeJson(json)

    try:
        json = generateJson(inputFile)
        #print("First JSON: ")
        #print(str(json).replace('\'', '\"'))
        json = organizeJson(json)
        #print("Second JSON: ")
        #print(str(json).replace('\'', '\"'))
        # sys.exit()
    except KeyError:
        print("ERROR: The spreadsheet template is not correct. Check the sheet and column names are correct.")
        sys.exit()

    mapping_writer.writeValues(json,global_config.tmpDir)

    outputFile = mapping_writer.writeFinalFile(outputFile, json['TriplesMap'].keys(), json['Function'].keys())
    return outputFile



import pandas
import tempfile
import re
import sys
import requests


def checkFile(path):
    """
    Checks if the input 'path' is an excel file an can be correctly read, returns a boolean
    """
    try:
        data = pandas.ExcelFile(path, engine='openpyxl')
        return True
    except:
        return False


def gdriveToXMLX(url):
    temp = tempfile.NamedTemporaryFile(prefix="mapeathor-gdrive", delete=False, suffix=".xlsx")

    m = re.search(r'(?:file|spreadsheets)\/d\/(.*)\/', url)

    if not m:
        raise Exception("Malformed Google Spreadsheets URL")
        sys.exit()

    docid = m.groups()[0]

    url = 'https://docs.google.com/spreadsheets/d/'+docid+'/export?exportFormat=xlsx'

    r = requests.get(url)
    temp.write(r.content)
    return temp.name



import os
import re 

from setuptools import setup, find_packages


# borrowed from SQLAlchemy
with open(os.path.join(os.path.dirname(__file__), 'src', 'mapeathor', '_version.py')) as file:
    version = (re.compile(r""".*__version__ = ["'](.*?)['"]""", re.S).match(file.read()).group(1))

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt") as r:
    requirements = r.read().split("\n")[0:-1]

setup(
    name="mapeathor",
    version=version,
    author="Ana Iglesias-Molina",
    author_email="ana.iglesiasm@upm.es",
    license="Apache 2.0",
    description="Mapeathor is a simple spreadsheet parser able to generate mapping rules in three mapping languages: R2RML, RML (with extension to functions from FnO) and YARRRML.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/oeg-upm/Mapeathor",
    project_urls={
        'Documentation': 'https://github.com/oeg-upm/Mapeathor/wiki',
        'Source code': 'https://github.com/oeg-upm/Mapeathor/tree/master/src/mapeathor',
        'Issue tracker': 'https://github.com/oeg-upm/Mapeathor/issues',
    },
    include_package_data=True,
    packages=find_packages('src'),
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    install_requires=requirements,
    python_requires='>=3.6',
)
